#
# Copyright 2021 Splunk Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

import os.path as op


class AlertActionsTemplateMgr:
    CURRENT_DIR = op.dirname(op.abspath(__file__))
    DEFAULT_TEMPLATE_DIR = op.join(CURRENT_DIR, "arf_template")
    DEFAULT_HTML_LOOKUP_DIR = op.join(DEFAULT_TEMPLATE_DIR, "default_html_theme")

    def __init__(self, html_theme=None):
        self._html_theme = html_theme or AlertActionsTemplateMgr.DEFAULT_HTML_LOOKUP_DIR

    def get_html_lookup_dir(self):
        return self._html_theme



#
# Copyright 2021 Splunk Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from typing import Any, Dict, List, Set, Optional, Union
from dataclasses import dataclass
from splunk_add_on_ucc_framework.commands.openapi_generator.object_to_json import Init

# https://spec.openapis.org/oas/latest.html#data-types
# type	    format	    Comments
# integer	int32	    signed 32 bits
# integer	int64	    signed 64 bits (a.k.a long)
# number	float
# number	double
# string	password	A hint to UIs to obscure input.

OPENAPI_300 = "3.0.0"


@dataclass
class ServerVariableObject(
    Init
):  # https://spec.openapis.org/oas/latest.html#server-variable-object
    # enum: Set[str]  # cannot be empty
    default: str  # REQUIRED
    description: Optional[str] = None


@dataclass
class ServerObject(Init):  # https://spec.openapis.org/oas/latest.html#server-object
    url: str
    variables: Dict[str, ServerVariableObject]
    description: Optional[str] = None


@dataclass
class XMLObject(Init):  # https://spec.openapis.org/oas/latest.html#xml-object
    name: Optional[str] = None
    namespace: Optional[str] = None
    prefix: Optional[str] = None
    attribute: Optional[bool] = False
    wrapped: Optional[bool] = False


@dataclass
class SchemaObject(Init):  # https://spec.openapis.org/oas/latest.html#schema-object
    # discriminator	Discriminator Object
    # xml	XML Object
    # externalDocs	External Documentation Object
    # example	Any
    # above is a theory
    # below is practice
    type: Optional[str] = None
    properties: Optional[dict] = None
    items: Optional[Dict[str, str]] = None
    xml: Optional[XMLObject] = None


@dataclass
class ExampleObject(Init):
    pass


@dataclass
class EncodingObject(Init):
    pass


@dataclass
class MediaTypeObject(
    Init
):  # https://spec.openapis.org/oas/latest.html#media-type-object
    schema: Optional[Union[SchemaObject, Dict]] = None
    example: Optional[Any] = None
    examples: Optional[Dict[str, ExampleObject]] = None
    encoding: Optional[Dict[str, EncodingObject]] = None


@dataclass
class RequestBodyObject(
    Init
):  # https://spec.openapis.org/oas/latest.html#request-body-object
    content: Dict[str, MediaTypeObject]
    description: Optional[str] = None
    required: Optional[bool] = False


@dataclass
class ExternalDocumentationObject(Init):
    pass


@dataclass
class ParameterObject(Init):
    pass


@dataclass
class HeaderObject(Init):
    pass


@dataclass
class LinkObject(Init):
    pass


@dataclass
class ResponseObject(Init):  # https://spec.openapis.org/oas/latest.html#response-object
    description: str
    headers: Optional[Dict[str, HeaderObject]] = None
    content: Optional[Dict[str, MediaTypeObject]] = None
    links: Optional[Dict[str, LinkObject]] = None


# @dataclass
# class ResponsesObject(Init):
#   pass


@dataclass
class CallbackObjects(Init):
    pass


# @dataclass
# class SecurityRequirementObject(Init):
#   pass


@dataclass
class OperationObject(
    Init
):  # https://spec.openapis.org/oas/latest.html#operation-object
    # responses: ResponsesObject = None
    responses: Dict[str, ResponseObject]  # required by openapi3
    tags: Optional[Set[str]] = None
    summary: Optional[str] = None
    description: Optional[str] = None
    externalDocs: Optional[ExternalDocumentationObject] = None
    operationId: Optional[str] = None
    parameters: Optional[ParameterObject] = None
    requestBody: Optional[RequestBodyObject] = None
    callbacks: Optional[CallbackObjects] = None
    deprecated: Optional[bool] = False
    # security: List[SecurityRequirementObject] = None
    security: Optional[List[dict]] = None

    servers: Optional[ServerObject] = None


@dataclass
class PathItemObject(
    Init
):  # https://spec.openapis.org/oas/latest.html#path-item-object
    _ref: Optional[str] = None
    summary: Optional[str] = None
    get: Optional[OperationObject] = None
    # put: OperationObject = None
    post: Optional[OperationObject] = None
    delete: Optional[OperationObject] = None
    # options: OperationObject = None
    # head: OperationObject = None
    # patch: OperationObject = None
    # trace: OperationObject = None
    servers: Optional[ServerObject] = None
    parameters: Optional[List[Dict[str, Any]]] = None
    description: Optional[str] = None


# @dataclass
# class PathsObject(Init):  # https://spec.openapis.org/oas/latest.html#paths-object
#   pass
# not implementable


@dataclass
class ContactObject(Init):  # https://spec.openapis.org/oas/latest.html#contact-object
    name: Optional[str] = None
    url: Optional[str] = None
    email: Optional[str] = None


@dataclass
class LicenseObject(Init):  # https://spec.openapis.org/oas/latest.html#licenseObject
    name: str
    identifier: Optional[str] = None
    url: Optional[str] = None


@dataclass
class InfoObject(Init):  # https://spec.openapis.org/oas/latest.html#infoObject
    title: str
    version: str
    summary: Optional[str] = None
    description: Optional[str] = None
    termsOfService: Optional[str] = None
    contact: Optional[ContactObject] = None
    license: Optional[LicenseObject] = None


@dataclass
class SecuritySchemeObjects(
    Init
):  # https://spec.openapis.org/oas/latest.html#security-scheme-object
    type: str
    # name : str
    # in: str
    scheme: str
    # bearerFormat: str
    # flows: OAuthFlowsObject
    # openIdConnectUrl: str


@dataclass
class LinkObjects(Init):
    pass


@dataclass
class TagObject(Init):
    pass


@dataclass
class ComponentsObject(
    Init
):  # https://spec.openapis.org/oas/latest.html#components-object
    schemas: Optional[Dict[str, SchemaObject]] = None
    responses: Optional[Dict[str, ResponseObject]] = None
    parameters: Optional[Dict[str, ParameterObject]] = None
    examples: Optional[Dict[str, ExampleObject]] = None
    requestBodies: Optional[Dict[str, RequestBodyObject]] = None
    headers: Optional[Dict[str, HeaderObject]] = None
    securitySchemes: Optional[Dict[str, SecuritySchemeObjects]] = None
    links: Optional[Dict[str, LinkObjects]] = None
    callbacks: Optional[Dict[str, CallbackObjects]] = None
    pathItems: Optional[Dict[str, PathItemObject]] = None


@dataclass
class OpenAPIObject(Init):  # https://spec.openapis.org/oas/latest.html#openapi-object
    openapi: str
    info: InfoObject
    jsonSchemaDialect: Optional[str] = None
    servers: Optional[List[ServerObject]] = None
    webhooks: Optional[Dict[str, PathItemObject]] = None
    components: Optional[ComponentsObject] = None
    tags: Optional[List[TagObject]] = None
    externalDocs: Optional[ExternalDocumentationObject] = None
    # paths: Optional[PathsObject] = None
    # security: List[SecurityRequirementObject] = None
    # despite above follows strict definition,
    # below needs to be used, to be implementable
    paths: Optional[Dict[str, PathItemObject]] = None
    security: Optional[List[dict]] = None



#
# Copyright 2021 Splunk Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from typing import Sequence

import addonfactory_splunk_conf_parser_lib as conf_parser

SERVER_CONF_FILE_NAME = "server.conf"


class ServerConf:
    def __init__(self):
        self._server_conf = conf_parser.TABConfigParser()

    def create_default(self, conf_file_names: Sequence[str]):
        self._server_conf.add_section("shclustering")
        for conf_file_name in conf_file_names:
            self._server_conf["shclustering"][
                f"conf_replication_include.{conf_file_name}"
            ] = "true"

    def write(self, path: str) -> None:
        with open(path, "w") as fd:
            self._server_conf.write(fd)



#
# Copyright 2021 Splunk Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
import logging
import os
import shutil

from splunk_add_on_ucc_framework.commands.modular_alert_builder.alert_actions_conf_gen import (
    generate_alert_actions_conf,
)
from splunk_add_on_ucc_framework.commands.modular_alert_builder.alert_actions_html_gen import (
    generate_alert_actions_html_files,
)
from splunk_add_on_ucc_framework.commands.modular_alert_builder.alert_actions_py_gen import (
    generate_alert_actions_py_files,
)

logger = logging.getLogger("ucc_gen")


def _copy_alert_icon_to_output(
    internal_root_dir: str,
    output_directory: str,
    addon_name: str,
):
    shutil.copy(
        os.path.join(internal_root_dir, "static", "alerticon.png"),
        os.path.join(output_directory, addon_name, "appserver", "static"),
    )


def generate_alerts(internal_source_dir: str, output_dir: str, envs):
    output_content = {}
    conf_return = None
    html_return = None
    py_return = None
    global_settings = envs["global_settings"]

    _copy_alert_icon_to_output(internal_source_dir, output_dir, envs["product_id"])
    package_dir = os.path.join(output_dir, envs["product_id"])

    build_components = envs["build_components"]
    if build_components["conf"]:
        conf_return = generate_alert_actions_conf(
            input_setting=envs["schema.content"],
            package_path=package_dir,
            global_settings=global_settings,
        )

    if build_components["html"]:
        html_return = generate_alert_actions_html_files(
            input_setting=envs["schema.content"],
            package_path=package_dir,
            html_setting=envs["html_setting"],
        )

    if build_components["py"]:
        py_return = generate_alert_actions_py_files(
            input_setting=envs["schema.content"],
            package_path=package_dir,
            global_settings=global_settings,
        )

    if conf_return:
        output_content["conf"] = conf_return
    if html_return:
        output_content["html"] = conf_return
    if py_return:
        output_content["py"] = py_return

    return output_content



#
# Copyright 2021 Splunk Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
import logging
import re
from os import path as op

from jinja2 import Environment, FileSystemLoader

from splunk_add_on_ucc_framework.commands.modular_alert_builder import (
    alert_actions_exceptions as aae,
)
from splunk_add_on_ucc_framework.commands.modular_alert_builder import (
    arf_consts as ac,
)
from splunk_add_on_ucc_framework.commands.modular_alert_builder.alert_actions_helper import (
    write_file,
)

logger = logging.getLogger("ucc_gen")


class AlertActionsPyBase:
    def __init__(
        self, input_setting=None, package_path=None, global_settings=None, **kwargs
    ):
        self._all_setting = input_setting
        self._package_path = package_path
        self._global_settings = global_settings
        self._current_alert = None
        self._alert_actions_setting = input_setting[ac.MODULAR_ALERTS]
        self._ta_name = self._all_setting.get(ac.SHORT_NAME)
        self._lib_dir = self.get_python_lib_dir_name(self._ta_name)
        # nosemgrep: splunk.autoescape-disabled
        self._templates = Environment(
            loader=FileSystemLoader(
                op.join(op.dirname(op.realpath(__file__)), "arf_template")
            ),
            trim_blocks=True,
            lstrip_blocks=True,
            keep_trailing_newline=True,
        )

    def get_python_lib_dir_name(self, app_name):
        space_replace = re.compile(r"[^\w]+")
        return space_replace.sub("_", app_name.lower())

    def get_alert_py_name(self, helper=""):
        return self._current_alert[ac.SHORT_NAME] + helper + ".py"

    def get_alert_py_path(self):
        if not self._package_path:
            return None
        return op.join(self._package_path, "bin", self.get_alert_py_name())

    def get_alert_helper_py_name(self):
        return "modalert_" + self._current_alert[ac.SHORT_NAME] + "_helper.py"

    def get_alert_helper_py_path(self):
        if not self._package_path:
            return None
        return op.join(
            self._package_path, "bin", self._lib_dir, self.get_alert_helper_py_name()
        )

    def get_template_py_files(self):
        bin_dir = op.join(self._package_path, "bin")
        return [
            op.join(bin_dir, self._lib_dir + "_declare.pyc"),
            op.join(bin_dir, self._lib_dir + "_declare.pyo"),
            op.join(bin_dir, self._lib_dir, "setup_util_helper.py"),
            op.join(bin_dir, self._lib_dir, "setup_util_helper.pyc"),
            op.join(bin_dir, self._lib_dir, "setup_util_helper.pyo"),
        ]


class AlertActionsPyGenerator(AlertActionsPyBase):
    DEFAULT_TEMPLATE_PY = "alert_action.py.template"
    DEFAULT_TEMPLATE_HELPER_PY = "alert_action_helper.py.template"

    def __init__(
        self,
        input_setting=None,
        package_path=None,
        template_py=None,
        template_helper_py=None,
        global_settings=None,
        **kwargs
    ):
        if not input_setting:
            msg = 'required_args="input_setting""'
            raise aae.AlertActionsInValidArgs(msg)
        super().__init__(
            input_setting=input_setting,
            package_path=package_path,
            template_py=template_py,
            global_settings=global_settings,
            **kwargs
        )

        self._template = None
        self._template_py = template_py or AlertActionsPyGenerator.DEFAULT_TEMPLATE_PY
        self._template_helper_py = (
            template_helper_py or AlertActionsPyGenerator.DEFAULT_TEMPLATE_HELPER_PY
        )
        logger.info("template_py=%s", self._template_py)
        self._output = {}

    def merge_py_code(self, init, new):
        if not init:
            logger.info("No previous code, don't merge new parameters in")
            return new

        start = r"\[sample_code_macro:start\]"
        end = r"\[sample_code_macro:end\]"
        start_str = "[sample_code_macro:start]"
        end_str = "[sample_code_macro:end]"
        pattern = re.compile(start + r"((.|[\r\n])*)" + end, re.MULTILINE)
        matched = pattern.search(init)
        if not matched:
            logger.info("No macro anymore, don't merge new parameters in")
            return init

        matched = pattern.search(new)
        if matched:
            new_c = matched.group(1)
            return re.sub(
                start + r"((.|[\r\n])*)" + end, start_str + new_c + end_str, init
            )

    def gen_py_file(self, one_alert_setting):
        self._current_alert = one_alert_setting
        self.gen_main_py_file()
        self.gen_helper_py_file()

    def gen_main_py_file(self):
        template = self._templates.get_template(self._template_py)

        settings = None
        if self._global_settings:
            settings = self._global_settings["settings"]
        rendered_content = template.render(
            input=self._all_setting,
            lib_name=self._lib_dir,
            mod_alert=self._current_alert,
            global_settings=settings,
            helper_name=op.splitext(self.get_alert_helper_py_name())[0],
        )

        logger.debug('operation="Writing file", file="%s"', self.get_alert_py_path())
        write_file(
            self.get_alert_py_name(),
            self.get_alert_py_path(),
            rendered_content,
        )
        name = self._current_alert[ac.SHORT_NAME]
        if not self._output.get(name):
            self._output[name] = {}
        self._output[name][self.get_alert_py_name()] = rendered_content

    def gen_helper_py_file(self):
        template = self._templates.get_template(self._template_helper_py)

        name = self._current_alert[ac.SHORT_NAME]
        init_content = None
        if self._current_alert.get("code"):
            init_content = self._current_alert.get("code")

        settings = {}
        if self._global_settings:
            settings = self._global_settings.get("settings", {})
        rendered_content = template.render(
            input=self._all_setting,
            mod_alert=self._current_alert,
            global_settings=settings,
        )

        final_content = self.merge_py_code(init_content, rendered_content)
        logger.debug('operation="Writing file", file="%s"', self.get_alert_py_path())
        write_file(
            self.get_alert_helper_py_name(),
            self.get_alert_helper_py_path(),
            final_content,
        )
        if not self._output.get(name):
            self._output[name] = {}
        self._output[name][self.get_alert_py_name(helper="_helper")] = final_content

    def handle(self):
        for alert in self._alert_actions_setting:
            logger.info(
                'operation="Generate py file", alert_action="%s"', alert[ac.SHORT_NAME]
            )
            self.gen_py_file(alert)


def generate_alert_actions_py_files(
    input_setting=None, package_path=None, global_settings=None, **kwargs
):
    py_gen = AlertActionsPyGenerator(
        input_setting=input_setting,
        package_path=package_path,
        global_settings=global_settings,
        **kwargs
    )
    py_gen.handle()
    return py_gen._output



#
# Copyright 2021 Splunk Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
import logging
import os
import shutil
import stat
from pathlib import Path
from typing import Sequence

logger = logging.getLogger("ucc_gen")


class SplunktaucclibNotFound(Exception):
    pass


def _check_ucc_library_in_requirements_file(path_to_requirements: str) -> bool:
    with open(path_to_requirements) as f_reqs:
        content = f_reqs.readlines()
    for line in content:
        if "splunktaucclib" in line:
            return True
    return False


def install_python_libraries(
    path: str, ucc_lib_target: str, python_binary_name: str, includes_ui: bool = False
):
    if os.path.isfile(os.path.join(path, "lib", "requirements.txt")):
        path_to_reqs_file = os.path.join(path, "lib", "requirements.txt")
    elif os.path.isfile(
        os.path.join(os.path.abspath(os.path.join(path, os.pardir)), "requirements.txt")
    ):
        path_to_reqs_file = os.path.join(
            os.path.abspath(os.path.join(path, os.pardir)), "requirements.txt"
        )
    else:
        path_to_reqs_file = None
    if path_to_reqs_file is not None:
        logger.info(f"Installing requirements from {path_to_reqs_file}")
        if includes_ui:
            ucc_library_present = _check_ucc_library_in_requirements_file(
                path_to_reqs_file
            )
            if not ucc_library_present:
                raise SplunktaucclibNotFound(
                    f"splunktaucclib is not found in {path_to_reqs_file}. "
                    f"Please add it there because this add-on has UI."
                )
        install_libraries(
            path_to_reqs_file,
            ucc_lib_target,
            python_binary_name,
        )
    else:
        logger.info("Could not find requirements file, nothing to install")

    packages_to_remove = (
        "setuptools",
        "bin",
        "pip",
        "distribute",
        "wheel",
    )
    remove_package_from_installed_path(
        ucc_lib_target,
        packages_to_remove,
    )

    remove_execute_bit(ucc_lib_target)


def install_libraries(
    requirements_file_path: str,
    installation_path: str,
    installer: str,
):
    if not os.path.isfile(requirements_file_path):
        logger.warning(f"Unable to find requirements file: {requirements_file_path}")
    else:
        if not os.path.isdir(installation_path):
            os.makedirs(installation_path)
        install_cmd = (
            installer
            + ' -m pip install -r "'
            + requirements_file_path
            + '" --no-compile --prefer-binary --ignore-installed --use-deprecated=legacy-resolver --target "'
            + installation_path
            + '"'
        )
        os.system(installer + " -m pip install pip --upgrade")
        os.system(install_cmd)


def remove_package_from_installed_path(
    installation_path: str, package_names: Sequence[str]
):
    p = Path(installation_path)
    try:
        for package_name in package_names:
            for o in p.glob(f"{package_name}*"):
                if o.is_dir():
                    logger.info(f"  removing directory {o} from {installation_path}")
                    shutil.rmtree(o)
    except FileNotFoundError:
        pass


def remove_execute_bit(installation_path: str):
    p = Path(installation_path)
    no_user_exec = ~stat.S_IEXEC
    no_group_exec = ~stat.S_IXGRP
    no_other_exec = ~stat.S_IXOTH
    no_exec = no_user_exec & no_group_exec & no_other_exec

    for o in p.rglob("*"):
        if not o.is_dir() and os.access(o, os.X_OK):
            logger.info(f"  fixing {o} execute bit")
            current_permissions = stat.S_IMODE(os.lstat(o).st_mode)
            os.chmod(o, current_permissions & no_exec)



#
# Copyright 2021 Splunk Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from typing import Any, Dict


class DataClasses:
    def __init__(self, json: Dict) -> None:
        self._json = json
        self.__dict__.update(self._iterator(self._json))
        #   __dict__ contains references to base object
        #   update, do not override!

    def __getattr__(self, name: str):
        _name = f"_{name}"
        if _name in self.__dict__:
            return self.__dict__[_name]
        else:
            raise AttributeError()
            #   hasattr expects AttributeError exception

    def _iteration_manager(self, element: Any) -> Any:
        if isinstance(element, dict):
            return DataClasses(element)
        elif isinstance(element, list):
            return self._list_iterator(element)
        else:
            return element

    def _list_iterator(self, _list: list) -> list:
        return_list = []
        for i in _list:
            return_list.append(self._iteration_manager(i))
        return return_list

    def _iterator(self, json: Any):
        d = json
        for k, v in json.items():
            d[k] = self._iteration_manager(v)
        return d



#
# Copyright 2021 Splunk Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
import json
from typing import Any, Dict

import dunamai
import yaml

from splunk_add_on_ucc_framework import exceptions


def dump_json_config(config: Dict[Any, Any], file_path: str):
    with open(file_path, "w") as f:
        json.dump(config, f, ensure_ascii=False, indent=4)
        f.write("\n")


def dump_yaml_config(config: Dict[Any, Any], file_path: str):
    with open(file_path, "w") as f:
        yaml.dump(config, f, indent=4)


def get_version_from_git():
    version = dunamai.Version.from_git()
    if not version.stage:
        stage = "R"
    else:
        stage = version.stage[:1]
    try:
        version.serialize(metadata=True, style=dunamai.Style.SemVer)
    except ValueError:
        raise exceptions.CouldNotVersionFromGitException()
    return f"{version.base}{stage}{version.commit}"



#
# Copyright 2021 Splunk Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
import os
import os.path as op


class GlobalConfigPostProcessor:
    """
    Post process for REST builder.
    """

    output_local = "local"
    _import_declare_template = """
import {import_declare_name}
"""

    _import_declare_content = """
import os
import sys
import re
from os.path import dirname

ta_name = '{ta_name}'
pattern = re.compile(r'[\\\\/]etc[\\\\/]apps[\\\\/][^\\\\/]+[\\\\/]bin[\\\\/]?$')
new_paths = [path for path in sys.path if not pattern.search(path) or ta_name in path]
new_paths.append(os.path.join(dirname(dirname(__file__)), "lib"))
new_paths.insert(0, os.path.sep.join([os.path.dirname(__file__), ta_name]))
sys.path = new_paths
"""

    def __init__(self):
        self.builder = None
        self.schema = None
        self.import_declare_name = "import_declare_test"

    @property
    def root_path(self):
        return getattr(self.builder.output, "_path")

    def import_declare_py_name(self):
        if self.import_declare_name:
            return self.import_declare_name
        return f"{self.schema.namespace}_import_declare"

    def import_declare_py_content(self):
        import_declare_file = op.join(
            self.root_path,
            self.builder.output.bin,
            self.import_declare_py_name() + ".py",
        )
        content = self._import_declare_content.format(
            ta_name=self.schema.product,
        )
        with open(import_declare_file, "w") as f:
            f.write(content)

    def import_declare(self, rh_file):
        with open(rh_file) as f:
            cont = f.readlines()
        import_declare = self._import_declare_template.format(
            import_declare_name=self.import_declare_py_name()
        )
        cont.insert(0, import_declare)
        with open(rh_file, "w") as f:
            f.write("".join(cont))

    def __call__(self, builder, schema):
        """
        :param builder: REST builder
        :param schema: Global Config Schema
        :return:
        """
        self.builder = builder
        self.schema = schema

        self.import_declare_py_content()
        for endpoint in schema.endpoints:
            rh_file = op.join(
                getattr(builder.output, "_path"),
                builder.output.bin,
                endpoint.rh_name + ".py",
            )
            self.import_declare(rh_file)

        # add executable permission to files under bin folder
        def add_executable_attribute(file_path):
            if op.isfile(file_path):
                st = os.stat(file_path)
                os.chmod(file_path, st.st_mode | 0o111)

        bin_path = op.join(getattr(builder.output, "_path"), builder.output.bin)
        items = os.listdir(bin_path)
        list(map(add_executable_attribute, items))



#
# Copyright 2021 Splunk Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from typing import List

from splunk_add_on_ucc_framework.commands.rest_builder.endpoint.base import (
    RestEndpointBuilder,
)

"""
This class is used to generate the endpoint for getting access token from
auth code and extends RestEndpointBuilder class
"""


class OAuthModelEndpointBuilder(RestEndpointBuilder):

    """
    This is initialization of this endpoint builder class
    """

    def __init__(self, name, j2_env, namespace, **kwargs):
        super().__init__(name, namespace, **kwargs)
        self._app_name = kwargs.get("app_name")
        self.j2_env = j2_env

    """
    Action will return the possible action for the endpoint
    """

    def actions(self) -> List[str]:
        return ["edit"]

    """
    This will actually populate the jinja template with the token values and return it
    """

    def generate_rh(self) -> str:
        return self.j2_env.get_template("oauth.template").render(
            app_name=self._app_name,
        )



#
# Copyright 2021 Splunk Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

import json
from typing import Dict, List
import warnings

APP_MANIFEST_FILE_NAME = "app.manifest"
APP_MANIFEST_WEBSITE = "https://dev.splunk.com/enterprise/reference/packagingtoolkit/pkgtoolkitappmanifest/"

DEPRECATION_MESSAGE = f"""
Comments are not allowed in app.manifest file.
Please refer to {APP_MANIFEST_WEBSITE}
"""


class AppManifestFormatException(Exception):
    pass


class AppManifest:
    def __init__(self):
        self._manifest = None
        self._comments = []

    def get_addon_name(self) -> str:
        return self._manifest["info"]["id"]["name"]

    def get_title(self) -> str:
        return self._manifest["info"]["title"]

    def get_description(self) -> str:
        return self._manifest["info"]["description"]

    def get_license_name(self) -> str:
        return self._manifest["info"]["license"]["name"]

    def get_license_uri(self) -> str:
        return self._manifest["info"]["license"]["uri"]

    def get_authors(self) -> List[Dict[str, str]]:
        return self._manifest["info"]["author"]

    @property
    def manifest(self) -> Dict:
        return self._manifest

    def read(self, content: str) -> None:
        try:
            self._manifest = json.loads(content)
        except json.JSONDecodeError:
            # Manifest file has comments.
            manifest_lines = []
            for line in content.split("\n"):
                if line.lstrip().startswith("#"):
                    self._comments.append(line)
                else:
                    manifest_lines.append(line)
            if self._comments:
                warnings.warn(DEPRECATION_MESSAGE, FutureWarning)
            manifest = "".join(manifest_lines)
            try:
                self._manifest = json.loads(manifest)
            except json.JSONDecodeError:
                raise AppManifestFormatException

    def update_addon_version(self, version: str) -> None:
        self._manifest["info"]["id"]["version"] = version

    def __str__(self) -> str:
        content = json.dumps(self._manifest, indent=4, sort_keys=True)
        if self._comments:
            for comment in self._comments:
                content += f"\n{comment}"
        return content



#
# Copyright 2021 Splunk Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
import copy
from functools import lru_cache
from typing import List, Dict, Any, Optional, Union
from splunk_add_on_ucc_framework import global_config as global_config_lib
from splunk_add_on_ucc_framework import app_manifest as app_manifest_lib
from splunk_add_on_ucc_framework.commands.openapi_generator.oas import (
    OpenAPIObject,
)
from splunk_add_on_ucc_framework.commands.openapi_generator import oas, json_to_object
from splunk_add_on_ucc_framework.commands.openapi_generator.json_to_object import (
    DataClasses,
)


def __create_min_required(
    global_config: global_config_lib.GlobalConfig,
) -> OpenAPIObject:
    return OpenAPIObject(
        openapi=oas.OPENAPI_300,
        info=oas.InfoObject(title=global_config.product, version=global_config.version),
        paths={},
    )


def __add_info_object_details(
    open_api_object: OpenAPIObject,
    global_config: global_config_lib.GlobalConfig,
    app_manifest: app_manifest_lib.AppManifest,
) -> OpenAPIObject:
    open_api_object.info.description = global_config.display_name
    if app_manifest.get_license_name():
        open_api_object.info.license = oas.LicenseObject(
            name=app_manifest.get_license_name(), url=app_manifest.get_license_uri()
        )
    author = app_manifest.get_authors()[0]
    open_api_object.info.contact = oas.ContactObject(
        name=author["name"], email=author["email"]
    )
    return open_api_object


def __add_server_object(
    open_api_object: OpenAPIObject, global_config: global_config_lib.GlobalConfig
) -> OpenAPIObject:
    description = "Access via management interface"
    default_domain = "localhost"
    default_port = "8089"

    variables = {
        "domain": oas.ServerVariableObject(default=default_domain),
        "port": oas.ServerVariableObject(default=default_port),
    }
    server = oas.ServerObject(
        url=f"https://{{domain}}:{{port}}/servicesNS/-/{global_config.product}",
        description=description,
        variables=variables,
    )

    open_api_object.servers = [server]
    return open_api_object


def __add_security_scheme_object(open_api_object: OpenAPIObject) -> OpenAPIObject:
    if open_api_object.components is not None:
        open_api_object.components.securitySchemes = {
            "BasicAuth": oas.SecuritySchemeObjects(type="http", scheme="basic")
        }
    return open_api_object


def __get_schema_object(*, name: str, entities: list) -> oas.SchemaObject:
    schema_object = oas.SchemaObject(
        type="object", xml=oas.XMLObject(name=name), properties={}
    )
    for entity in entities:
        if "helpLink" == entity.type:
            continue
        if schema_object.properties is not None:
            schema_object.properties[entity.field] = {"type": "string"}
            if hasattr(entity, "options") and hasattr(
                entity.options, "autoCompleteFields"
            ):
                field_values = [
                    autoCompleteField.value
                    if hasattr(autoCompleteField, "value")
                    else None
                    for autoCompleteField in entity.options.autoCompleteFields
                ]
                from_children = [
                    child.value
                    for autocomplete_field in entity.options.autoCompleteFields
                    if hasattr(autocomplete_field, "children")
                    for child in autocomplete_field.children
                ]
                schema_object.properties[entity.field]["enum"] = (
                    field_values + from_children
                )
            if hasattr(entity, "encrypted") and entity.encrypted:
                schema_object.properties[entity.field]["format"] = "password"
    return schema_object


def __add_schemas_object(
    open_api_object: OpenAPIObject, global_config: DataClasses
) -> OpenAPIObject:
    if open_api_object.components is not None:
        open_api_object.components.schemas = {}
        for tab in global_config.pages.configuration.tabs:
            open_api_object.components.schemas[tab.name] = __get_schema_object(
                name=tab.name, entities=tab.entity
            )
        additional_input_entities = [
            json_to_object.DataClasses(
                json={
                    "field": "disabled",
                    "type": "singleSelect",
                    "options": {
                        "autoCompleteFields": [
                            {"value": "0"},
                            {"value": "1"},
                        ]
                    },
                }
            )
        ]
        if hasattr(global_config.pages, "inputs") and hasattr(
            global_config.pages.inputs, "services"
        ):
            for service in global_config.pages.inputs.services:
                open_api_object.components.schemas[service.name] = __get_schema_object(
                    name=service.name,
                    entities=service.entity + additional_input_entities,
                )
    return open_api_object


#   consider changing to 'cache' once python is upgraded to >=3.9
@lru_cache(maxsize=None)
def __get_media_type_object_with_schema_ref(
    *, schema_name: str, schema_type: Optional[str] = None, is_xml: bool = False
) -> oas.MediaTypeObject:
    ref_dict = {"$ref": f"#/components/schemas/{schema_name}"}
    schema: Union[oas.SchemaObject, Dict] = (
        oas.SchemaObject(
            type=schema_type,
            items=ref_dict,
            xml=oas.XMLObject(name=f"{schema_name}_list", wrapped=True)
            if is_xml
            else None,
        )
        if schema_type
        else ref_dict
    )
    return oas.MediaTypeObject(schema=schema)


def __get_path_get(
    *, name: str, description: str, schema_type: Optional[str] = None
) -> oas.OperationObject:
    return oas.OperationObject(
        description=description,
        responses={
            "200": oas.ResponseObject(
                description=description,
                content={
                    "application/json": __get_media_type_object_with_schema_ref(
                        schema_name=name, schema_type=schema_type
                    ),
                    "application/xml": __get_media_type_object_with_schema_ref(
                        schema_name=name, schema_type=schema_type, is_xml=True
                    ),
                },
            )
        },
    )


def __get_path_get_for_list(*, name: str) -> oas.OperationObject:
    description = f"Get list of items for {name}"
    return __get_path_get(name=name, description=description, schema_type="array")


def __get_path_get_for_item(*, name: str) -> oas.OperationObject:
    description = f"Get {name} item details"
    return __get_path_get(name=name, description=description)


def __get_path_post(*, name: str, description: str) -> oas.OperationObject:
    return oas.OperationObject(
        description=description,
        requestBody=oas.RequestBodyObject(
            content={
                "application/x-www-form-urlencoded": __get_media_type_object_with_schema_ref(
                    schema_name=name
                )
            }
        ),
        responses={
            "200": oas.ResponseObject(
                description=description,
                content={
                    "application/json": __get_media_type_object_with_schema_ref(
                        schema_name=name
                    ),
                    "application/xml": __get_media_type_object_with_schema_ref(
                        schema_name=name, is_xml=True
                    ),
                },
            )
        },
    )


def __get_path_post_for_create(*, name: str) -> oas.OperationObject:
    description = f"Create item in {name}"
    return __get_path_post(name=name, description=description)


def __get_path_post_for_update(*, name: str) -> oas.OperationObject:
    description = f"Update {name} item"
    return __get_path_post(name=name, description=description)


def __get_path_delete(*, name: str) -> oas.OperationObject:
    description = f"Delete {name} item"
    return oas.OperationObject(
        description=description,
        responses={
            "200": oas.ResponseObject(
                description=description,
                content={
                    "application/json": __get_media_type_object_with_schema_ref(
                        schema_name=name, schema_type="array"
                    ),
                    "application/xml": __get_media_type_object_with_schema_ref(
                        schema_name=name, schema_type="array", is_xml=True
                    ),
                },
            )
        },
    )


def __get_output_mode() -> Dict[str, Any]:
    return {
        "name": "output_mode",
        "in": "query",
        "required": False,
        "description": "The name of the item to operate on",
        "schema": {"type": "string", "enum": ["xml", "json"]},
    }


def __assign_ta_paths(
    *,
    open_api_object: OpenAPIObject,
    path: str,
    path_name: str,
    actions: List[str],
):
    if open_api_object.paths is not None:
        open_api_object.paths[path] = oas.PathItemObject(
            get=__get_path_get_for_list(name=path_name),
            post=__get_path_post_for_create(name=path_name),
        )
        open_api_object.paths[path].parameters = [
            __get_output_mode(),
        ]
        if actions is not None and "clone" in actions:
            path = f"{path}/{{name}}"
            open_api_object.paths[path] = oas.PathItemObject(
                get=__get_path_get_for_item(name=path_name),
                post=__get_path_post_for_update(name=path_name),
                delete=__get_path_delete(name=path_name)
                if "delete" in actions
                else None,
            )
            open_api_object.paths[path].parameters = [
                {
                    "name": "name",
                    "in": "path",
                    "required": True,
                    "description": "The name of the item to operate on",
                    "schema": {"type": "string"},
                },
                __get_output_mode(),
            ]
    return open_api_object


def __add_paths(
    open_api_object: OpenAPIObject, global_config: DataClasses
) -> OpenAPIObject:
    for tab in global_config.pages.configuration.tabs:
        open_api_object = __assign_ta_paths(
            open_api_object=open_api_object,
            path=f"/{global_config.meta.restRoot}_{tab.name}"
            if hasattr(tab, "table")
            else f"/{global_config.meta.restRoot}_settings/{tab.name}",
            path_name=tab.name,
            actions=tab.table.actions
            if hasattr(tab, "table") and hasattr(tab.table, "actions")
            else None,
        )
    if hasattr(global_config.pages, "inputs") and hasattr(
        global_config.pages.inputs, "services"
    ):
        for service in global_config.pages.inputs.services:
            open_api_object = __assign_ta_paths(
                open_api_object=open_api_object,
                path=f"/{global_config.meta.restRoot}_{service.name}",
                path_name=service.name,
                actions=global_config.pages.inputs.table.actions,
            )
    return open_api_object


def transform(
    global_config: global_config_lib.GlobalConfig,
    app_manifest: app_manifest_lib.AppManifest,
) -> OpenAPIObject:
    content_copy = copy.deepcopy(global_config.content)
    global_config_dot_notation = json_to_object.DataClasses(json=content_copy)

    open_api_object = __create_min_required(global_config)
    open_api_object = __add_info_object_details(
        open_api_object,
        global_config,
        app_manifest,
    )
    open_api_object = __add_server_object(open_api_object, global_config)
    open_api_object.components = oas.ComponentsObject()
    open_api_object = __add_security_scheme_object(open_api_object)
    open_api_object.security = [{"BasicAuth": []}]
    open_api_object = __add_schemas_object(open_api_object, global_config_dot_notation)
    open_api_object = __add_paths(open_api_object, global_config_dot_notation)
    return open_api_object



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['splunk_add_on_ucc_framework',
 'splunk_add_on_ucc_framework.commands',
 'splunk_add_on_ucc_framework.commands.init_template.{{cookiecutter.addon_name}}.package.bin',
 'splunk_add_on_ucc_framework.commands.modular_alert_builder',
 'splunk_add_on_ucc_framework.commands.openapi_generator',
 'splunk_add_on_ucc_framework.commands.rest_builder',
 'splunk_add_on_ucc_framework.commands.rest_builder.endpoint']

package_data = \
{'': ['*'],
 'splunk_add_on_ucc_framework': ['package/appserver/static/js/*',
                                 'package/appserver/static/js/build/*',
                                 'package/appserver/templates/*',
                                 'package/default/data/ui/nav/*',
                                 'package/default/data/ui/views/*',
                                 'schema/*',
                                 'static/*',
                                 'templates/*'],
 'splunk_add_on_ucc_framework.commands': ['init_template/*',
                                          'init_template/{{cookiecutter.addon_name}}/*',
                                          'init_template/{{cookiecutter.addon_name}}/package/*',
                                          'init_template/{{cookiecutter.addon_name}}/package/lib/*'],
 'splunk_add_on_ucc_framework.commands.modular_alert_builder': ['arf_template/*',
                                                                'arf_template/default_html_theme/*']}

install_requires = \
['PyYAML>=6.0,<7.0',
 'addonfactory-splunk-conf-parser-lib>=0.3.3,<0.4.0',
 'cookiecutter>=2.1.1,<3.0.0',
 'dunamai>=1.9.0,<2.0.0',
 'jinja2>=2,<4',
 'jsonschema>=4.4.0,<5.0.0',
 'mkdocs-material>=9.0.15,<10.0.0',
 'openapi3>=1.7.0,<2.0.0']

entry_points = \
{'console_scripts': ['ucc-gen = splunk_add_on_ucc_framework.main:main']}

setup_kwargs = {
    'name': 'splunk-add-on-ucc-framework',
    'version': '5.23.0',
    'description': 'Splunk Add-on SDK formerly UCC is a build and code generation framework',
    'long_description': "# splunk-add-on-ucc-framework\n\n![PyPI](https://img.shields.io/pypi/v/splunk-add-on-ucc-framework)\n![Python](https://img.shields.io/pypi/pyversions/splunk-add-on-ucc-framework.svg)\n\n## What is UCC?\n\nUCC stands for  Universal Configuration Console. It is a service for generating Splunk Add-ons which is easily customizable and flexible.\nUCC provides basic UI template for creating Addon's UI. It is helpful to control the activity by using hooks and other functionalities.\n\n## Usage\n\nFor full usage instructions, please visit the [documentation](https://splunk.github.io/addonfactory-ucc-generator/).\n\n## Communication channels\n\nIf you are a Splunker use: https://splunk.slack.com/archives/C03T8QCHBTJ\n\nIf you are a part of the community use: https://splunk-usergroups.slack.com/archives/C03SG3ZL4S1\n\n## Contributing Guidelines\n\nGo to [Contributing Guidelines](https://splunk.github.io/addonfactory-ucc-generator/contributing/).\n",
    'author': 'Splunk',
    'author_email': 'addonfactory@splunk.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/splunk/addonfactory-ucc-generator',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)



#
# Copyright 2021 Splunk Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
import logging
import os
from os import linesep
from os import path as op
from re import search

from jinja2 import Environment, FileSystemLoader

from splunk_add_on_ucc_framework.commands.modular_alert_builder import (
    alert_actions_exceptions as aae,
)
from splunk_add_on_ucc_framework.commands.modular_alert_builder import (
    arf_consts as ac,
)
from splunk_add_on_ucc_framework.commands.modular_alert_builder.alert_actions_helper import (
    write_file,
)
from splunk_add_on_ucc_framework.commands.modular_alert_builder.alert_actions_template import (
    AlertActionsTemplateMgr,
)

logger = logging.getLogger("ucc_gen")


class AlertHtmlBase:
    def __init__(self, input_setting=None, package_path=None):
        self._all_setting = input_setting
        self._package_path = package_path
        self._current_alert = None
        # nosemgrep: splunk.autoescape-disabled
        self._templates = Environment(
            loader=FileSystemLoader(
                [
                    op.join(
                        op.dirname(op.realpath(__file__)),
                        "arf_template",
                        "default_html_theme",
                    ),
                    op.join(op.dirname(op.realpath(__file__)), "arf_template"),
                ]
            ),
            trim_blocks=True,
            lstrip_blocks=True,
            keep_trailing_newline=True,
        )

    def get_alert_html_name(self):
        return self._current_alert[ac.SHORT_NAME] + ".html"

    def get_alert_html_path(self):
        if not self._package_path:
            return None

        html_path = op.join(self._package_path, "default", "data", "ui", "alerts")
        if not op.exists(html_path):
            os.makedirs(html_path)

        return op.join(html_path, self.get_alert_html_name())


class AlertHtmlGenerator(AlertHtmlBase):
    DEFAULT_TEMPLATE_HTML = "mod_alert.html.template"
    DEFAULT_HOME_HTML = "default.html"

    def __init__(
        self,
        input_setting=None,
        package_path=None,
        html_template=None,
        html_home=None,
        html_theme=None,
    ):
        super().__init__(input_setting, package_path)
        if not input_setting:
            msg = 'required_args="input_setting"'
            raise aae.AlertActionsInValidArgs(msg)

        self._alert_actions_setting = input_setting[ac.MODULAR_ALERTS]
        self._html_template = html_template or AlertHtmlGenerator.DEFAULT_TEMPLATE_HTML
        self._html_home = html_home or AlertHtmlGenerator.DEFAULT_HOME_HTML
        self._temp_obj = AlertActionsTemplateMgr(html_theme=html_theme)
        self._html_theme = self._temp_obj.get_html_lookup_dir()
        logger.info(
            'html_theme="%s" html_template="%s", html_home="%s"',
            self._html_theme,
            self._html_template,
            self._html_home,
        )
        self._output = {}

    def handle_one_alert(self, template, one_alert_setting):
        self._current_alert = one_alert_setting
        final_form = template.render(
            mod_alert=self._current_alert, home_page=self._html_home
        )
        text = linesep.join(
            [s for s in final_form.splitlines() if not search(r"^\s*$", s)]
        )

        logger.debug(
            'operation="Write", object_type="File", object="%s"',
            self.get_alert_html_path(),
        )

        write_file(
            self.get_alert_html_name(),
            self.get_alert_html_path(),
            text,
        )
        self._output[self._current_alert["short_name"]] = text

    def handle(self):
        logger.info("html_theme=%s", self._html_theme)
        template = self._templates.get_template(self._html_template)
        logger.info("Start to generate alert actions html files")
        for alert in self._alert_actions_setting:
            self.handle_one_alert(template, alert)
        logger.info("Finished generating alert actions html files")


def generate_alert_actions_html_files(
    input_setting=None, package_path=None, html_setting=None
):
    html_template = None
    html_home = None
    html_theme = None
    if html_setting:
        html_template = html_setting.get("html_template")
        html_home = html_setting.get("html_home")
        html_theme = html_setting.get("html_theme")
    html_gen = AlertHtmlGenerator(
        input_setting=input_setting,
        package_path=package_path,
        html_template=html_template,
        html_home=html_home,
        html_theme=html_theme,
    )
    html_gen.handle()
    return html_gen._output



#
# Copyright 2021 Splunk Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
import configparser
import json
import logging
import os
import shutil
import sys
from typing import Optional

from jinja2 import Environment, FileSystemLoader
from openapi3 import OpenAPI

from splunk_add_on_ucc_framework import (
    __version__,
    exceptions,
    global_config_update,
    global_config_validator,
    utils,
)
from splunk_add_on_ucc_framework import app_conf as app_conf_lib
from splunk_add_on_ucc_framework import meta_conf as meta_conf_lib
from splunk_add_on_ucc_framework import server_conf as server_conf_lib
from splunk_add_on_ucc_framework import app_manifest as app_manifest_lib
from splunk_add_on_ucc_framework import global_config as global_config_lib
from splunk_add_on_ucc_framework import normalize
from splunk_add_on_ucc_framework.commands.modular_alert_builder import (
    generate_alerts,
)
from splunk_add_on_ucc_framework.commands.rest_builder import (
    global_config_builder_schema,
    global_config_post_processor,
)
from splunk_add_on_ucc_framework.commands.rest_builder.builder import RestBuilder
from splunk_add_on_ucc_framework.install_python_libraries import (
    SplunktaucclibNotFound,
    install_python_libraries,
)
from splunk_add_on_ucc_framework.commands.openapi_generator import (
    ucc_to_oas,
)


logger = logging.getLogger("ucc_gen")

PARENT_DIR = ".."
internal_root_dir = os.path.dirname(os.path.dirname(__file__))
# nosemgrep: splunk.autoescape-disabled
j2_env = Environment(
    loader=FileSystemLoader(os.path.join(internal_root_dir, "templates"))
)


def _recursive_overwrite(src, dest, ignore_list=None):
    """
    Method to copy from src to dest recursively.

    Args:
        src (str): Source of copy
        dest (str): Destination to copy
        ignore_list (list): List of files/folder to ignore while copying
    """

    if os.path.isdir(src):
        if not os.path.isdir(dest):
            os.makedirs(dest)
        files = os.listdir(src)
        for f in files:
            if not ignore_list or not os.path.join(dest, f) in ignore_list:
                _recursive_overwrite(
                    os.path.join(src, f), os.path.join(dest, f), ignore_list
                )
            else:
                logger.info(f"Excluding : {os.path.join(dest, f)}")
    else:
        if os.path.exists(dest):
            os.remove(dest)
        shutil.copy(src, dest)


def _replace_token(ta_name, outputdir):
    """
    Replace token with addon name in inputs.xml, configuration.xml, redirect.xml.
    Replace token with addon version in redirect.xml.

    Args:
        ta_name (str): Name of TA.
        outputdir (str): output directory.
    """

    # replace token in template
    logger.info("Replace tokens in views")
    views = ["inputs.xml", "configuration.xml", "redirect.xml"]
    for view in views:
        template_dir = os.path.join(
            outputdir, ta_name, "default", "data", "ui", "views"
        )
        with open(os.path.join(template_dir, view)) as f:
            s = f.read()

        # Safely write the changed content, if found in the file
        with open(os.path.join(template_dir, view), "w") as f:
            s = s.replace("${package.name}", ta_name)
            if view == "redirect.xml":
                s = s.replace("${ta.name}", ta_name.lower())
            f.write(s)


def _generate_rest(
    ta_name,
    scheme: global_config_builder_schema.GlobalConfigBuilderSchema,
    outputdir,
):
    """
    Build REST for Add-on.

    Args:
        ta_name (str): Name of TA.
        scheme (GlobalConfigBuilderSchema): REST schema.
        outputdir (str): output directory.
    """
    builder_obj = RestBuilder(scheme, os.path.join(outputdir, ta_name))
    builder_obj.build()
    post_process = global_config_post_processor.GlobalConfigPostProcessor()
    post_process(builder_obj, scheme)
    return builder_obj


def _is_oauth_configured(ta_tabs):
    """
    Check if oauth is configured in globalConfig file.

    Args:
        ta_tabs (list): List of tabs mentioned in globalConfig file.

    Returns:
        bool: True if oauth is configured, False otherwise.
    """

    for tab in ta_tabs:
        if tab["name"] == "account":
            for elements in tab["entity"]:
                if elements["type"] == "oauth":
                    return True
            break
    return False


def _replace_oauth_html_template_token(ta_name, ta_version, outputdir):
    """
    Replace tokens with addon name and version in redirect.html.

    Args:
        ta_name (str): Name of TA.
        ta_version (str): Version of TA.
        outputdir (str): output directory.
    """

    html_template_path = os.path.join(outputdir, ta_name, "appserver", "templates")
    with open(os.path.join(html_template_path, "redirect.html")) as f:
        s = f.read()

    # Safely write the changed content, if found in the file
    with open(os.path.join(html_template_path, "redirect.html"), "w") as f:
        # replace addon name in html template
        s = s.replace("${ta.name}", ta_name.lower())
        # replace addon version in html template
        s = s.replace("${ta.version}", ta_version)
        f.write(s)


def _modify_and_replace_token_for_oauth_templates(
    ta_name: str, global_config: global_config_lib.GlobalConfig, outputdir: str
):
    """
    Rename templates with respect to addon name if OAuth is configured.

    Args:
        ta_name: Add-on name.
        global_config: Object representing globalConfig.
        outputdir: output directory.
    """
    redirect_xml_src = os.path.join(
        outputdir, ta_name, "default", "data", "ui", "views", "redirect.xml"
    )
    redirect_js_src = os.path.join(
        outputdir, ta_name, "appserver", "static", "js", "build", "redirect_page.js"
    )
    redirect_html_src = os.path.join(
        outputdir, ta_name, "appserver", "templates", "redirect.html"
    )

    if _is_oauth_configured(global_config.tabs):
        _replace_oauth_html_template_token(ta_name, global_config.version, outputdir)

        redirect_js_dest = (
            os.path.join(outputdir, ta_name, "appserver", "static", "js", "build", "")
            + ta_name.lower()
            + "_redirect_page."
            + global_config.version
            + ".js"
        )
        redirect_html_dest = os.path.join(
            outputdir,
            ta_name,
            "appserver",
            "templates",
            ta_name.lower() + "_redirect.html",
        )
        redirect_xml_dest = os.path.join(
            outputdir,
            ta_name,
            "default",
            "data",
            "ui",
            "views",
            ta_name.lower() + "_redirect.xml",
        )
        os.rename(redirect_js_src, redirect_js_dest)
        os.rename(redirect_html_src, redirect_html_dest)
        os.rename(redirect_xml_src, redirect_xml_dest)

    # if oauth is not configured remove the extra template
    else:
        os.remove(redirect_xml_src)
        os.remove(redirect_html_src)
        os.remove(redirect_js_src)


def _add_modular_input(
    ta_name: str, global_config: global_config_lib.GlobalConfig, outputdir: str
):
    """
    Generate Modular input for addon.

    Args:
        ta_name: Add-on name.
        global_config: Object representing globalConfig.
        outputdir: output directory.
    """
    for service in global_config.inputs:
        input_name = service.get("name")
        class_name = input_name.upper()
        description = service.get("title")
        entity = service.get("entity")
        field_allow_list = ["name", "index", "sourcetype"]
        template = "input.template"
        # if the service has a template specified, use it.  Otherwise keep the default
        if "template" in service:
            template = service.get("template") + ".template"

        # filter fields in allow list
        entity = [x for x in entity if x.get("field") not in field_allow_list]
        import_declare = "import import_declare_test"

        content = j2_env.get_template(template).render(
            import_declare=import_declare,
            input_name=input_name,
            class_name=class_name,
            description=description,
            entity=entity,
        )
        input_file_name = os.path.join(outputdir, ta_name, "bin", input_name + ".py")
        with open(input_file_name, "w") as input_file:
            input_file.write(content)

        input_default = os.path.join(outputdir, ta_name, "default", "inputs.conf")
        config = configparser.ConfigParser()
        if os.path.exists(input_default):
            config.read(input_default)

        if config.has_section(input_name):
            config[input_name]["python.version"] = "python3"
        else:
            config[input_name] = {"python.version": "python3"}

        with open(input_default, "w") as configfile:
            config.write(configfile)


def _make_modular_alerts(
    ta_name: str, global_config: global_config_lib.GlobalConfig, outputdir: str
):
    """
    Generate the alert schema with required structure.

    Args:
        ta_name: Add-on name.
        global_config: Object representing globalConfig.
        outputdir: Output directory.
    """
    if global_config.has_alerts():
        envs = normalize.normalize(
            {
                "alerts": global_config.alerts,
            },
            ta_name,
            global_config.namespace,
        )
        logger.info("Generating alerts code")
        generate_alerts(internal_root_dir, outputdir, envs)


def _get_ignore_list(ta_name, path):
    """
    Return path of files/folders to be removed.

    Args:
        ta_name (str): Name of TA.
        path (str): Path of '.uccignore'.

    Returns:
        list: List of paths to be removed from output directory.
    """
    if not os.path.exists(path):
        return []
    else:
        with open(path) as ignore_file:
            ignore_list = ignore_file.readlines()
        ignore_list = [
            (os.path.join("output", ta_name, _get_os_path(path))).strip()
            for path in ignore_list
        ]
        return ignore_list


def _remove_listed_files(ignore_list):
    """
    Return path of files/folders to removed in output folder.

    Args:
        ignore_list (list): List of files/folder to removed in output directory.

    """
    for path in ignore_list:
        if os.path.exists(path):
            if os.path.isfile(path):
                os.remove(path)
            elif os.path.isdir(path):
                shutil.rmtree(path, ignore_errors=True)
        else:
            logger.info(
                "While ignoring the files mentioned in .uccignore {} was not found".format(
                    path
                )
            )


def _handle_no_inputs(ta_name, outputdir):
    """
    Handle for configuration without input page.

    Args:
        ta_name (str): Name of TA.
        outputdir (str): output directory.
    """
    default_xml_file = os.path.join(
        outputdir, ta_name, "default", "data", "ui", "nav", "default.xml"
    )
    default_no_input_xml_file = os.path.join(
        outputdir, ta_name, "default", "data", "ui", "nav", "default_no_input.xml"
    )
    os.remove(default_xml_file)
    os.rename(
        default_no_input_xml_file,
        default_xml_file,
    )
    file_remove_list = [
        os.path.join(
            outputdir, ta_name, "default", "data", "ui", "views", "inputs.xml"
        ),
        os.path.join(outputdir, ta_name, "appserver", "static", "css", "inputs.css"),
        os.path.join(
            outputdir, ta_name, "appserver", "static", "css", "createInput.css"
        ),
    ]
    for fl in file_remove_list:
        try:
            os.remove(fl)
        except OSError:
            pass


def _get_os_path(path):
    """
    Returns a path which will be os compatible.

    Args:
        path (str): Path in string

    Return:
        string: Path which will be os compatible.
    """

    if "\\\\" in path:
        path = path.replace("\\\\", os.sep)
    else:
        path = path.replace("\\", os.sep)
    path = path.replace("/", os.sep)
    return path.strip(os.sep)


def _get_addon_version(addon_version: Optional[str]) -> str:
    if not addon_version:
        try:
            return utils.get_version_from_git()
        except exceptions.CouldNotVersionFromGitException:
            logger.error(
                "Could not find the proper version from git tags. "
                "Check out "
                "https://github.com/splunk/addonfactory-ucc-generator/issues/404"
            )
            exit(1)
    return addon_version.strip()


def generate(
    source: str,
    config_path: Optional[str] = None,
    addon_version: Optional[str] = None,
    output_directory: Optional[str] = None,
    python_binary_name: str = "python3",
):
    logger.info(f"ucc-gen version {__version__} is used")
    logger.info(f"Python binary name to use: {python_binary_name}")
    if output_directory is None:
        output_directory = os.path.join(os.getcwd(), "output")
    addon_version = _get_addon_version(addon_version)
    logger.info(f"Add-on will be built with version '{addon_version}'")
    if not os.path.exists(source):
        raise NotADirectoryError(f"{os.path.abspath(source)} not found.")
    shutil.rmtree(os.path.join(output_directory), ignore_errors=True)
    os.makedirs(os.path.join(output_directory))
    logger.info(f"Cleaned out directory {output_directory}")
    app_manifest_path = os.path.abspath(
        os.path.join(source, app_manifest_lib.APP_MANIFEST_FILE_NAME),
    )
    with open(app_manifest_path) as manifest_file:
        app_manifest_content = manifest_file.read()
    app_manifest = app_manifest_lib.AppManifest()
    try:
        app_manifest.read(app_manifest_content)
    except app_manifest_lib.AppManifestFormatException:
        logger.error(
            f"Manifest file @ {app_manifest_path} has invalid format.\n"
            f"Please refer to {app_manifest_lib.APP_MANIFEST_WEBSITE}.\n"
            f'Lines with comments are supported if they start with "#".\n'
        )
        sys.exit(1)
    ta_name = app_manifest.get_addon_name()
    if not config_path:
        is_global_config_yaml = False
        config_path = os.path.abspath(os.path.join(source, "..", "globalConfig.json"))
        if not os.path.isfile(config_path):
            config_path = os.path.abspath(
                os.path.join(source, "..", "globalConfig.yaml")
            )
            is_global_config_yaml = True
    else:
        is_global_config_yaml = True if config_path.endswith(".yaml") else False

    if os.path.isfile(config_path):
        logger.info(f"Using globalConfig file located @ {config_path}")
        global_config = global_config_lib.GlobalConfig()
        global_config.parse(config_path, is_global_config_yaml)
        try:
            validator = global_config_validator.GlobalConfigValidator(
                internal_root_dir, global_config
            )
            validator.validate()
            logger.info("globalConfig file is valid")
        except global_config_validator.GlobalConfigValidatorException as e:
            logger.error(f"globalConfig file is not valid. Error: {e}")
            sys.exit(1)
        global_config.update_addon_version(addon_version)
        global_config.dump(global_config.original_path)
        logger.info(
            f"Updated and saved add-on version in the globalConfig file to {addon_version}"
        )
        global_config_update.handle_global_config_update(global_config)
        scheme = global_config_builder_schema.GlobalConfigBuilderSchema(
            global_config, j2_env
        )
        _recursive_overwrite(
            os.path.join(internal_root_dir, "package"),
            os.path.join(output_directory, ta_name),
        )
        logger.info("Copied UCC template directory")
        global_config_file = (
            "globalConfig.yaml" if is_global_config_yaml else "globalConfig.json"
        )
        output_global_config_path = os.path.join(
            output_directory,
            ta_name,
            "appserver",
            "static",
            "js",
            "build",
            global_config_file,
        )
        shutil.copyfile(
            config_path,
            output_global_config_path,
        )
        logger.info("Copied globalConfig to output")
        ucc_lib_target = os.path.join(output_directory, ta_name, "lib")
        try:
            install_python_libraries(
                source, ucc_lib_target, python_binary_name, includes_ui=True
            )
        except SplunktaucclibNotFound as e:
            logger.error(str(e))
            sys.exit(1)
        logger.info(
            f"Installed add-on requirements into {ucc_lib_target} from {source}"
        )
        _replace_token(ta_name, output_directory)
        _generate_rest(ta_name, scheme, output_directory)
        _modify_and_replace_token_for_oauth_templates(
            ta_name,
            global_config,
            output_directory,
        )
        if global_config.has_inputs():
            default_no_input_xml_file = os.path.join(
                output_directory,
                ta_name,
                "default",
                "data",
                "ui",
                "nav",
                "default_no_input.xml",
            )
            os.remove(default_no_input_xml_file)
            _add_modular_input(ta_name, global_config, output_directory)
        else:
            _handle_no_inputs(ta_name, output_directory)
        _make_modular_alerts(ta_name, global_config, output_directory)

        conf_file_names = []
        conf_file_names.extend(list(scheme.settings_conf_file_names))
        conf_file_names.extend(list(scheme.configs_conf_file_names))
        conf_file_names.extend(list(scheme.oauth_conf_file_names))

        source_server_conf_path = os.path.join(source, "default", "server.conf")
        # For now, only create server.conf only if no server.conf is present in
        # the source package.
        if not os.path.isfile(source_server_conf_path):
            server_conf = server_conf_lib.ServerConf()
            server_conf.create_default(conf_file_names)
            output_server_conf_path = os.path.join(
                output_directory,
                ta_name,
                "default",
                server_conf_lib.SERVER_CONF_FILE_NAME,
            )
            server_conf.write(output_server_conf_path)
            logger.info(
                f"Created default {server_conf_lib.SERVER_CONF_FILE_NAME} file in the output folder"
            )
    else:
        global_config = None
        conf_file_names = []
        logger.warning(
            "Skipped generating UI components as globalConfig file does not exist"
        )
        ucc_lib_target = os.path.join(output_directory, ta_name, "lib")
        install_python_libraries(source, ucc_lib_target, python_binary_name)
        logger.info(
            f"Installed add-on requirements into {ucc_lib_target} from {source}"
        )

    ignore_list = _get_ignore_list(
        ta_name, os.path.abspath(os.path.join(source, PARENT_DIR, ".uccignore"))
    )
    _remove_listed_files(ignore_list)
    if ignore_list:
        logger.info(f"Removed {ignore_list} files")
    _recursive_overwrite(source, os.path.join(output_directory, ta_name))
    logger.info("Copied package directory")

    default_meta_conf_path = os.path.join(
        output_directory, ta_name, "metadata", meta_conf_lib.DEFAULT_META_FILE_NAME
    )
    if not os.path.exists(default_meta_conf_path):
        os.makedirs(os.path.join(output_directory, ta_name, "metadata"), exist_ok=True)
        meta_conf = meta_conf_lib.MetaConf()
        meta_conf.create_default()
        meta_conf.write(default_meta_conf_path)
        logger.info(
            f"Created default {meta_conf_lib.DEFAULT_META_FILE_NAME} file in the output folder"
        )

    with open(os.path.join(output_directory, ta_name, "VERSION"), "w") as version_file:
        version_file.write(addon_version)
        version_file.write("\n")
        version_file.write(addon_version)
        logger.info("Updated VERSION file")

    app_manifest.update_addon_version(addon_version)
    output_manifest_path = os.path.abspath(
        os.path.join(output_directory, ta_name, app_manifest_lib.APP_MANIFEST_FILE_NAME)
    )
    with open(output_manifest_path, "w") as manifest_file:
        manifest_file.write(str(app_manifest))
        logger.info(
            f"Updated {app_manifest_lib.APP_MANIFEST_FILE_NAME} file in the output folder"
        )

    app_conf = app_conf_lib.AppConf()
    output_app_conf_path = os.path.join(
        output_directory, ta_name, "default", app_conf_lib.APP_CONF_FILE_NAME
    )
    app_conf.read(output_app_conf_path)
    app_conf.update(addon_version, app_manifest, conf_file_names)
    app_conf.write(output_app_conf_path)
    logger.info(f"Updated {app_conf_lib.APP_CONF_FILE_NAME} file in the output folder")

    license_dir = os.path.abspath(os.path.join(source, PARENT_DIR, "LICENSES"))
    if os.path.exists(license_dir):
        logger.info("Copy LICENSES directory")
        _recursive_overwrite(
            license_dir, os.path.join(output_directory, ta_name, "LICENSES")
        )

    if os.path.exists(
        os.path.abspath(os.path.join(source, PARENT_DIR, "additional_packaging.py"))
    ):
        sys.path.insert(0, os.path.abspath(os.path.join(source, PARENT_DIR)))
        from additional_packaging import additional_packaging

        additional_packaging(ta_name)

    if global_config:
        logger.info("Generating OpenAPI file")
        open_api_object = ucc_to_oas.transform(global_config, app_manifest)
        open_api = OpenAPI(open_api_object.json)

        output_openapi_folder = os.path.abspath(
            os.path.join(output_directory, ta_name, "static")
        )
        output_openapi_path = os.path.join(output_openapi_folder, "openapi.json")
        if not os.path.isdir(output_openapi_folder):
            os.makedirs(os.path.join(output_openapi_folder))
            logger.info(f"Creating {output_openapi_folder} folder")
        with open(output_openapi_path, "w") as openapi_file:
            json.dump(open_api.raw_element, openapi_file, indent=4)



#
# Copyright 2021 Splunk Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#


from splunk_add_on_ucc_framework.commands.rest_builder.endpoint.base import (
    indent,
    quote_string,
)


class RestFieldBuilder:

    _kv_template = "{name} = {value}"
    _rh_template = """field.RestField(
    {name},
    required={required},
    encrypted={encrypted},
    default={default},
    validator={validator}
)"""

    def __init__(self, name, required, encrypted, default, validator):
        self._name = name
        self._required = required
        self._encrypted = encrypted
        self._default = default
        self._validator = validator

    def generate_spec(self) -> str:
        return self._kv_template.format(
            name=self._name,
            value="",
        )

    def generate_conf_with_default_value(self) -> str:
        return self._kv_template.format(
            name=self._name, value=self._default if self._default is not None else ""
        )

    def _indent_validator(self) -> str:
        validator = indent(self._validator)
        return validator[4:]

    def generate_rh(self) -> str:
        return self._rh_template.format(
            name=quote_string(self._name),
            required=self._required,
            encrypted=self._encrypted,
            default=quote_string(self._default),
            validator=self._indent_validator(),
        )
